searchNodes=[{"ref":"ThousandIsland.html","title":"ThousandIsland","type":"module","doc":"Provides a high-level interface for starting and managing ThousandIsland.Server instances. Server instances exist as a process tree, which is usually started by starting this module&#39;s child spec within an existing supervision tree, or by calling start_link/1 directly. Examples A typical use of ThousandIsland might look like the following: defmodule MyApp.Supervisor do # ... other Supervisor boilerplate def init(config) do children = [ # ... other children as dictated by your app {ThousandIsland, port: 1234, handler_module: MyApp.ConnectionHandler} ] Supervisor.init(children, strategy: :one_for_one) end end You can also start servers directly via the start_link/1 function: {:ok, pid} = ThousandIsland.start_link(port: 1234, handler_module: MyApp.ConnectionHandler) Configuration A number of options are defined when starting a server. The complete list is defined by the t:options() type. Logging &amp; Telemetry As a low-level library, Thousand Island purposely does not do any inline logging of any kind. The ThousandIsland.Logging module defines a number of functions to aid in tracing connections at various log levels, and such logging can be dynamically enabled and disabled against an already running server. This logging is backed by :telemetry events internally, and if desired these events can also be hooked by your application for logging or metric purposes. The following is a complete list of events emitted by Thousand Island: [:listener, :start]: Emitted when the server first starts up. [:transport, :listen, :start]: Emitted when a transport module successfully listens on the configured port. [:acceptor, :start]: Emitted when an acceptor process starts up. [:acceptor, :accept]: Emitted when an acceptor process accepts a new client connection. [:acceptor, :shutdown]: Emitted when an acceptor process shuts down. [:connection, :handler, :start]: Emitted when a connection is started. [:connection, :handler, :complete]: Emitted when a connection completes normally. [:connection, :handler, :exception]: Emitted when a Handler raises an exception during processing. [:connection, :handler, :startup_timeout]: Emitted when the transfer of the underlying socket does not complete in time. Internal-only for the most part. [:connection, :handler, :handshake_error]: Emitted when the handshake fails, most often a result of the TLS library being unable to agree on certificates or ciphers. [:socket, :recv, :complete]: Emitted whenever a ThousandIsland.Socket.recv/3 call completes. [:socket, :send, :complete]: Emitted whenever a ThousandIsland.Socket.send/2 call completes. [:socket, :shutdown, :complete]: Emitted whenever a ThousandIsland.Socket.shutdown/2 call completes. [:socket, :close, :complete]: Emitted wheneverr a ThousandIsland.Socket.close/1 call completes. Where meaurements indicate a time duration they are are expressed in System :native units for performance reasons. They can be conveted to any desired time unit via System.convert_time_unit/3."},{"ref":"ThousandIsland.html#local_port/1","title":"ThousandIsland.local_port/1","type":"function","doc":"Returns the local port number that the servrer is listening on."},{"ref":"ThousandIsland.html#start_link/1","title":"ThousandIsland.start_link/1","type":"function","doc":"Starts a ThousandIsland.Server instance with the given options. Returns a pid that can be used to further manipulate the server via other functions defined on this module in the case of success, or an error tuple describing the reason the server was unable to start in the case of failure."},{"ref":"ThousandIsland.html#stop/2","title":"ThousandIsland.stop/2","type":"function","doc":"Synchronously stops the given server, waiting up to the given number of milliseconds for existing connections to finish up. Immediately upon calling this function, the server stops listening for new connections, and then proceeds to wait until either all existing connections have completed or the specified timeout has elapsed."},{"ref":"ThousandIsland.html#t:options/0","title":"ThousandIsland.options/0","type":"type","doc":"Possible options to configure a server. Valid option values are as follows: handler_module: The name of the module used to handle connections to this server. The module is expected to implement the ThousandIsland.Handler behaviour. Required. handler_options: A term which is passed as the second argument to all ThousandIsland.Handler.handle_connection/2 calls. Optional. port: The TCP port number to listen on. If not specified this defaults to 4000. If a port number of 0 is given, the server will dynamically assign a port number which can then be obtained via local_port/1. transport_module: The name of the module which provides basic socket functions. Thousand Island provides ThousandIsland.Transports.TCP and ThousandIsland.Transports.SSL, which provide clear and TLS encrypted TCP sockets respectively. If not specified this defaults to ThousandIsland.Transports.TCP. transport_options: A keyword list of options to be passed to the transport module&#39;s ThousandIsland.Transport.listen/2 function. Valid values depend on the transport module specified in transport_module and can be found in the documentation for the ThousandIsland.Transports.TCP and ThousandIsland.Transports.SSL modules. Any options in terms of interfaces to listen to / certificates and keys to use for SSL connections will be passed in via this option. num_acceptors: The numbner of acceptor processes to run. Defaults to 10."},{"ref":"ThousandIsland.Handler.html","title":"ThousandIsland.Handler","type":"behaviour","doc":"Defines the behaviour required of the application layer of a Thousand Island server. Users pass the name of a module implementing this behaviour as the handler_module parameter when starting a server instance, and Thousand Island will call this module&#39;s handle_connection/2 function every time a client connects to the server."},{"ref":"ThousandIsland.Handler.html#c:handle_connection/2","title":"ThousandIsland.Handler.handle_connection/2","type":"callback","doc":"Called by Thousand Island once for every client connection. This callback is called from a private process managed by Thousand Island, and has complete and sole control over the given socket. The socket can be written to, read from, and otherwise manipulated via the ThousandIsland.Socket interface. The socket will already have completed handshaking for protocols which require such a step (such as SSL). If the socket is not closed within the callback, Thousand Island will ensure that it is properly closed. The second argument consists of an arbitrary term specified via the handler_options parameter at server start time. The return value of this callback is unused."},{"ref":"ThousandIsland.Handlers.Daytime.html","title":"ThousandIsland.Handlers.Daytime","type":"module","doc":"A sample Handler implementation of the Daytime protocol https://en.wikipedia.org/wiki/Daytime_Protocol"},{"ref":"ThousandIsland.Handlers.Discard.html","title":"ThousandIsland.Handlers.Discard","type":"module","doc":"A sample Handler implementation of the Discard protocol https://en.wikipedia.org/wiki/Discard_Protocol"},{"ref":"ThousandIsland.Handlers.Echo.html","title":"ThousandIsland.Handlers.Echo","type":"module","doc":"A sample Handler implementation of the Echo protocol https://en.wikipedia.org/wiki/Echo_Protocol"},{"ref":"ThousandIsland.Handlers.HTTPHelloWorld.html","title":"ThousandIsland.Handlers.HTTPHelloWorld","type":"module","doc":"A sample Handler implementation of a simple HTTP Server. Intended to be the simplest thing that can answer a browser request and nothing more. Not even remotely strictly HTTP compliant."},{"ref":"ThousandIsland.Logger.html","title":"ThousandIsland.Logger","type":"module","doc":"Allows dynamically adding and altering the log level used to trace connections within a Thousand Island server via the use of telemetry hooks. Should you wish to do your own logging or tracking of these events, a complete list of the telemetry events emitted by Thousand Island is described in the module documentation for ThousandIsland."},{"ref":"ThousandIsland.Logger.html#attach_logger/1","title":"ThousandIsland.Logger.attach_logger/1","type":"function","doc":"Start logging Thousand Island at the specified log level. Valid values for log level are :error, :info, :debug, and :trace. Enabling a given log level implicitly enables all higher log levels as well."},{"ref":"ThousandIsland.Logger.html#detach_logger/1","title":"ThousandIsland.Logger.detach_logger/1","type":"function","doc":"Stop logging Thousand Island at the specified log level. Disabling a given log level implicitly disables all lower log levels as well."},{"ref":"ThousandIsland.Logger.html#log_debug/4","title":"ThousandIsland.Logger.log_debug/4","type":"function","doc":""},{"ref":"ThousandIsland.Logger.html#log_info/4","title":"ThousandIsland.Logger.log_info/4","type":"function","doc":""},{"ref":"ThousandIsland.Logger.html#log_trace/4","title":"ThousandIsland.Logger.log_trace/4","type":"function","doc":""},{"ref":"ThousandIsland.Socket.html","title":"ThousandIsland.Socket","type":"module","doc":"Encapsulates a client connection&#39;s underlying socket, providing a facility to read, write, and otherwise manipulate a connection from a client. ThousandIsland.Socket instances are passed to the application layer of a server via the ThousandIsland.Handler.handle_connection/2 callback."},{"ref":"ThousandIsland.Socket.html#close/1","title":"ThousandIsland.Socket.close/1","type":"function","doc":"Closes the given socket."},{"ref":"ThousandIsland.Socket.html#local_info/1","title":"ThousandIsland.Socket.local_info/1","type":"function","doc":"Returns information in the form of t:ThousandIsland.Transport.socket_info() about the local end of the socket."},{"ref":"ThousandIsland.Socket.html#peer_info/1","title":"ThousandIsland.Socket.peer_info/1","type":"function","doc":"Returns information in the form of t:ThousandIsland.Transport.socket_info() about the remote end of the socket."},{"ref":"ThousandIsland.Socket.html#recv/3","title":"ThousandIsland.Socket.recv/3","type":"function","doc":"Returns available bytes on the given socket. Up to num_bytes bytes will be returned (0 can be passed in to get the next &#39;available&#39; bytes, typically the next packet). If insufficient bytes are available, the functino can wait timeout milliseconds for data to arrive."},{"ref":"ThousandIsland.Socket.html#send/2","title":"ThousandIsland.Socket.send/2","type":"function","doc":"Sends the given data (specified as a binary or an IO list) on the given socket."},{"ref":"ThousandIsland.Socket.html#setopts/2","title":"ThousandIsland.Socket.setopts/2","type":"function","doc":"Sets the given flags on the socket"},{"ref":"ThousandIsland.Socket.html#shutdown/2","title":"ThousandIsland.Socket.shutdown/2","type":"function","doc":"Shuts down the socket in the given direction."},{"ref":"ThousandIsland.Socket.html#t:t/0","title":"ThousandIsland.Socket.t/0","type":"opaque","doc":"A reference to a socket along with metadata describing how to use it"},{"ref":"ThousandIsland.Transport.html","title":"ThousandIsland.Transport","type":"behaviour","doc":"This module describes the behaviour required for Thousand Island to interact with low-level sockets. It is largely internal to Thousand Island, however users are free to implement their own versions of this behaviour backed by whatever underlying transport they choose. Such a module can be used in Thousand Island by passing its name as the transport_module option when starting up a server, as described in ThousandIsland."},{"ref":"ThousandIsland.Transport.html#c:accept/1","title":"ThousandIsland.Transport.accept/1","type":"callback","doc":"Wait for a client connection on the given listener socket. This call blocks until such a connection arrives, or an error occurs (such as the listener socket being closed)."},{"ref":"ThousandIsland.Transport.html#c:close/1","title":"ThousandIsland.Transport.close/1","type":"callback","doc":"Closes the given socket."},{"ref":"ThousandIsland.Transport.html#c:controlling_process/2","title":"ThousandIsland.Transport.controlling_process/2","type":"callback","doc":"Transfers ownership of the given socket to the given process. This will always be called by the process which currently owns the socket."},{"ref":"ThousandIsland.Transport.html#c:handshake/1","title":"ThousandIsland.Transport.handshake/1","type":"callback","doc":"Performs an initial handshake on a new client connection (such as that done when negotiating an SSL connection). Transports which do not have such a handshake can simply pass the socket through unchanged."},{"ref":"ThousandIsland.Transport.html#c:listen/2","title":"ThousandIsland.Transport.listen/2","type":"callback","doc":"Create and return a listener socket bound to the given port and configured per the provided options."},{"ref":"ThousandIsland.Transport.html#c:listen_port/1","title":"ThousandIsland.Transport.listen_port/1","type":"callback","doc":"Return the local port number that the given listener socket is accepting connections on."},{"ref":"ThousandIsland.Transport.html#c:local_info/1","title":"ThousandIsland.Transport.local_info/1","type":"callback","doc":"Returns information in the form of t:socket_info() about the local end of the socket."},{"ref":"ThousandIsland.Transport.html#c:peer_info/1","title":"ThousandIsland.Transport.peer_info/1","type":"callback","doc":"Returns information in the form of t:socket_info() about the remote end of the socket."},{"ref":"ThousandIsland.Transport.html#c:recv/3","title":"ThousandIsland.Transport.recv/3","type":"callback","doc":"Returns available bytes on the given socket. Up to num_bytes bytes will be returned (0 can be passed in to get the next &#39;available&#39; bytes, typically the next packet). If insufficient bytes are available, the functino can wait timeout milliseconds for data to arrive."},{"ref":"ThousandIsland.Transport.html#c:send/2","title":"ThousandIsland.Transport.send/2","type":"callback","doc":"Sends the given data (specified as a binary or an IO list) on the given socket."},{"ref":"ThousandIsland.Transport.html#c:setopts/2","title":"ThousandIsland.Transport.setopts/2","type":"callback","doc":"Sets the given options on the socket. Should disallow setting of options which are not compatible with Thousand Island"},{"ref":"ThousandIsland.Transport.html#c:shutdown/2","title":"ThousandIsland.Transport.shutdown/2","type":"callback","doc":"Shuts down the socket in the given direction."},{"ref":"ThousandIsland.Transport.html#t:listener_socket/0","title":"ThousandIsland.Transport.listener_socket/0","type":"opaque","doc":"A listener socket used to wait for connections"},{"ref":"ThousandIsland.Transport.html#t:on_recv/0","title":"ThousandIsland.Transport.on_recv/0","type":"type","doc":"The return value from a recv/3 call"},{"ref":"ThousandIsland.Transport.html#t:socket/0","title":"ThousandIsland.Transport.socket/0","type":"opaque","doc":"A socket representing a client connection"},{"ref":"ThousandIsland.Transport.html#t:socket_info/0","title":"ThousandIsland.Transport.socket_info/0","type":"type","doc":"Information about an endpoint (either remote (&#39;peer&#39;) or local"},{"ref":"ThousandIsland.Transport.html#t:socket_options/0","title":"ThousandIsland.Transport.socket_options/0","type":"type","doc":"Options which can be set on a socket via setopts/2"},{"ref":"ThousandIsland.Transport.html#t:way/0","title":"ThousandIsland.Transport.way/0","type":"type","doc":"The direction in which to shutdown a connection in advance of closing it"},{"ref":"ThousandIsland.Transports.SSL.html","title":"ThousandIsland.Transports.SSL","type":"module","doc":"Defines a ThousandIsland.Transport implementation based on TCP SSL sockets as provided by Erlang&#39;s :ssl module. For the most part, users of Thousand Island will only ever need to deal with this module via transport_options passed to ThousandIsland at startup time. A complete list of such options is defined via the t::ssl.tls_server_option type. This list can be somewhat difficult to decipher; a list of the most common options follows: key: A DER encoded binary representation of the SSL key to use cert: A DER encoded binary representation of the SSL key to use keyfile: A string path to a PEM encoded key to use for SSL *certfile: A string path to a PEM encoded cert to use for SSL ip: The IP to listen on (defaults to all interfaces). IPs should be described in tuple form (ie: ip: {1, 2, 3, 4}). The value :loopback can be used to only bind to localhost. On platforms which support it (macOS and Linux at a minimum, likely others), you can also bind to a Unix domain socket by specifying a value of ip: {:local, &quot;/path/to/socket&quot;}. Note that the port must be set to 0, and that the socket is not removed from the filesystem after the server shuts down. Unless overridden, this module uses the following default options: backlog: 1024, nodelay: true, linger: {true, 30}, send_timeout: 30_000, send_timeout_close: true, reuseaddr: true The following options are required for the proper operation of Thousand Island and cannot be overridden: mode: :binary, active: false"},{"ref":"ThousandIsland.Transports.TCP.html","title":"ThousandIsland.Transports.TCP","type":"module","doc":"Defines a ThousandIsland.Transport implementation based on clear TCP sockets as provided by Erlang&#39;s :gen_tcp module. For the most part, users of Thousand Island will only ever need to deal with this module via transport_options passed to ThousandIsland at startup time. A complete list of such options is defined via the t::gen_tcp.listen_option() type. This list can be somewhat difficult to decipher; by far the most common value to pass to this transport is the following: ip: The IP to listen on (defaults to all interfaces). IPs should be described in tuple form (ie: ip: {1, 2, 3, 4}). The value :loopback can be used to only bind to localhost. On platforms which support it (macOS and Linux at a minimum, likely others), you can also bind to a Unix domain socket by specifying a value of ip: {:local, &quot;/path/to/socket&quot;}. Note that the port must be set to 0, and that the socket is not removed from the filesystem after the server shuts down. Unless overridden, this module uses the following default options: backlog: 1024, nodelay: true, linger: {true, 30}, send_timeout: 30_000, send_timeout_close: true, reuseaddr: true The following options are required for the proper operation of Thousand Island and cannot be overridden: mode: :binary, active: false"}]